<?php

namespace TDevAgency\FilesManager\Commands;

use Exception;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use TDevAgency\FilesManager\Helpers\CacheHelper;
use TDevAgency\FilesManager\Helpers\GenerateSizesHelper;
use TDevAgency\FilesManager\Helpers\SymlinkHelper;
use TDevAgency\FilesManager\Jobs\FilesManagerGenerateSizeJob;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

class GenerateSizesCommand extends Command
{
    protected $signature = 'fm:generate-sizes  {--id=} {--Q|queue}';

    protected $description = 'Generate and re-generate images from original one';

    /**
     * @return int
     * @throws Exception
     */
    public function handle(): int
    {
        $fields = [
            'id',
            'file_mime',
            'name',
            'original_name'
        ];
        if ($this->hasOption('id') && $this->option('id') !== null) {
            $files = TdfmFile::where('id', $this->option('id'))->get($fields);
        } else {
            $path = Storage::disk('tdfm_generated')->path('');
            File::cleanDirectory($path);
            $files = TdfmFile::get($fields);
        }

        if ($files->isEmpty()) {
            $this->info('Nothing to re-generate');
            return 0;
        }

        $bar = $this->output->createProgressBar($files->count());

        $bar->start();
        foreach ($files as $file) {
            $bar->advance();
            if ($this->hasOption('queue') && (bool)$this->option('queue') !== false) {
                dispatch(new FilesManagerGenerateSizeJob($file));
            } else {
                app(GenerateSizesHelper::class)->handle($file);
                app(SymlinkHelper::class)->symlinkFile($file);
                app(CacheHelper::class)->removeFileData($file->id);
            }
        }

        $bar->finish();
        return 0;
    }
}
